
#####################################################################
#
# xcrvfit function definitions 
#
#####################################################################


#####################################################################
# define the functions in xcrvfit 
#
proc init_fit_funcs { } {
	global myFunc xcv

#
# define the function groups
#
	set xcv(function,list) {common-nmr others experimental}

	set xcv(function,common-nmr) {
		T1_RELAX T2_RELAX R2_DIMER R2X_DIMER PH_TITR \
		XY1 XY2 GAUSS SUM_LOREN CRYSTAL}

	set xcv(function,others) { 
		LINE QUADRATIC CUBIC EXPON LORENTZ SUM_EXPON_1 \
		SUM_EXPON_2 SINU_1 SINU_2 SIGMOID }

	set xcv(function,experimental) {
		Z_DISP Z_AMIDE_PH Z_CHEM_MON_TRI Z_CHEM_TRI Z_CP_MONOMER Z_CP_DIMER \
		Z_CP_TRIMER Z_UCD Z_M_MELT Z_D_MELT Z_D_SIG}

#
# define each function, the 'key' variable is the same name used in
# fastcrvfit.
#
	set key					LINE
	set myFunc($key,name)	"Line" 
	set myFunc($key,note) 	"f(x) = a*x + b"
	set myFunc($key,parms)	"a b"	

	set key					QUADRATIC
	set myFunc($key,name)	"Quadratic" 
	set myFunc($key,note)	"f(x) = a*x*x + b*x + c"
	set myFunc($key,parms)	"a b c"	

	set key					CUBIC 
	set myFunc($key,name) 	"Cubic"
	set myFunc($key,note) 	"f(x) = a*x*x*x + b*x*x + c*x + d"
	set myFunc($key,parms) 	"a b c d"

	set key					EXPON
	set myFunc($key,name) 	"Exponential"
	set myFunc($key,note) 	"f(x) = a0 * EXP(a1 * x) + a2"
	set myFunc($key,parms) 	"a0 a1 a2"

	set key			 		SUM_EXPON_1
	set myFunc($key,name)	"Sum of Exponentials 1"
	set myFunc($key,note) 	"f(x) = a0*EXP(-x/a1) + a2*EXP(-x/a3)"
	set myFunc($key,parms) 	"a0 a1 a2 a3"

	set key			 		SUM_EXPON_2
	set myFunc($key,name) 	"Sum of Exponential 2"
	set myFunc($key,note) 	"f(x) = a0*(1-EXP(-x/a1))+a2*(1-EXP(-x/a3))"
	set myFunc($key,parms) 	"a0 a1 a2 a3"

	set key 				SINU_1
	set myFunc($key,name) 	"Sinusoidal 1"
	set myFunc($key,note)	"f(x) = a0 * cos(x)^2 - a1*cos(x) + a2"
	set myFunc($key,parms)	"a0 a1 a2"

	set key 				SINU_2
	set myFunc($key,name) 	"Sinusoidal 2"
	set myFunc($key,note)	"f(x) = a0 * cos(a1*x+a2) + a3"
	set myFunc($key,parms)	"a0 a1 a2 a3"

	set key 				SIGMOID
	set myFunc($key,name) 	"Sigmoidal"
	set myFunc($key,note) 	"f(x) = a + b / (1 + exp(-(x-c)/d))"
	set myFunc($key,parms) 	"a b c d"

# nmr functions

	set key					T1_RELAX 
	set myFunc($key,name) 	"T1 Relaxation"
	set myFunc($key,note) 	"f(x) = Mo*(1 - W*EXP(-x/T1))"
	set myFunc($key,parms) 	"Mo W T1"

	set key	 				T2_RELAX
	set myFunc($key,name) 	"T2 Relaxation"
	set myFunc($key,note) 	"f(x) = Mo*EXP(-x/T2)"
	set myFunc($key,parms) 	"Mo T2"
	
	set key				 	R2_DIMER
	set myFunc($key,name) 	"R2 Dimer"
	set myFunc($key,note) 	"f(x) = ((kd * b) + c + d) / 4x"
	set myFunc($key,parms) 	"kd R2D R2M"
	
	set key				 	R2X_DIMER
	set myFunc($key,name) 	"R2 Dimer + R2 Exchange"
	set myFunc($key,note) 	"f(x) = ((fm * r2m) + (fd * r2d) + (fm*fm) * fd * c)"
	set myFunc($key,parms) 	"kd R2D R2M TAU DSQU"

	set key			 		PH_TITR
	set myFunc($key,name) 	"PH Titration"
	set myFunc($key,note) 	"f(x) = Sa+(H**n/(H**n+Ka**n)) * (Sah-Sa)"
	set myFunc($key,parms) 	"pKa Sa Sah HC"

	set key 				XY1
	set myFunc($key,name) 	"Chemical Shift XY1"
	set myFunc($key,note) 	"Notes: X+Y <=> XY; obs X; vary Y"
	set myFunc($key,parms) 	"Shift Kd"

	set key 				XY2
	set myFunc($key,name) 	"Chemical Shift XY2"
	set myFunc($key,note)	"f(x) = X+Y<=>XY; XY+Y <=> XY2; obs X"
	set myFunc($key,parms)	"S_LM1 Kd1 S_LM2 Kd2"

	set key 				GAUSS
	set myFunc($key,name) 	"jFit Gaussian"
	set myFunc($key,note) 	"jFit Gaussian"
	set myFunc($key,parms) 	"Vo(1) T1(1) Vo(2) T1(2) Mo F"

	set key 				Z_DISP
	set myFunc($key,name) 	"jFit Dispersion"
	set myFunc($key,note) 	"jFit Dispersion"
	set myFunc($key,parms)	"Vo(1) T1(1) Vo(2) T1(2) Mo F"

	set key 				LORENTZ
	set myFunc($key,name)	"Lorentzian"
	set myFunc($key,note) 	"f(x) = Mo * (T1/(1+4*PI*PI*SQU((x-v0)*T1)"
	set myFunc($key,parms) 	"V0 T1 Mo"

	set key 				SUM_LOREN
	set myFunc($key,name)	"Sum of Lorentzians"
	set myFunc($key,note) 	"f(x) = Mo*(loren1 + F*loren2)"
	set myFunc($key,parms) 	"Vo(1) T1(1) Vo(2) T1(2) Mo F"

	set key 				Z_AMIDE_PH
	set myFunc($key,name) 	"Amide pH dependence"
	set myFunc($key,note) 	"Amide pH dependence"
	set myFunc($key,parms)	"T1 kOH kH kH2O"

	set key 				Z_UCD
	set myFunc($key,name) 	"UC_DIMER"
	set myFunc($key,note) 	"f(x) = m + a0**2*ka * exp(h*2M*(x*x-x0*x0))"
	set myFunc($key,parms) 	"a0 h M X0 Ka2"

	set key 				Z_M_MELT
	set myFunc($key,name) 	"Monomer Melt Curve"
	set myFunc($key,note) 	"Pierre Lavigne's monomer melt function"
	set myFunc($key,parms) 	"a0 a1 a2 a3 a4 a5 a6"

	set key 				Z_D_MELT
	set myFunc($key,name) 	"Dimer Melt Curve"
	set myFunc($key,note) 	"Pierre Lavigne's dimer melt function"
	set myFunc($key,parms) 	"a0 a1 a2 a3 a4 a5 a6 a7"

	set key 				Z_CHEM_TRI
	set myFunc($key,name)	"De-nature Trimer"
	set myFunc($key,note) 	"Pierre Lavigne's Chemical Denaturation of Trimer"
	set myFunc($key,parms) 	"a0 a1 a2 a3 a4 a5 a6 RT"
 
	set key 				Z_CHEM_MON_TRI
	set myFunc($key,name) 	"De-nature Monomer/Trimer"
	set myFunc($key,note)	"Pierre Lavigne's Chemical Denaturation of Monomer/Trimer"
	set myFunc($key,parms)	"a0 a1 a2 a3 a4 a5 a6 RT"

	set key 				Z_CP_MONOMER
	set myFunc($key,name)  	"Partial Molar Heat capacity of Monomer"
	set myFunc($key,note)	"Cp(temp) for unfolding of monomer"
	set myFunc($key,parms)	"a0 a1 a2 a3 a4 a5 a6 a7"

	set key 				Z_CP_DIMER
	set myFunc($key,name)  	"Partial Molar Heat capacity of Dimer"
	set myFunc($key,note)	"Cp(temp) for unfolding of dimer"
	set myFunc($key,parms)	"a0 a1 a2 a3 a4 a5 a6 a7 a8"

	set key 				Z_CP_TRIMER
	set myFunc($key,name)   	"Partial Molar Heat capacity of Trimer"
	set myFunc($key,note)	"Cp(temp) for unfolding of trimer"
	set myFunc($key,parms)	"a0 a1 a2 a3 a4 a5 a6 a7 a8"

	set key			 		Z_D_SIG
	set myFunc($key,name)	"Sigmoidal of dimer"
	set myFunc($key,note) 	"Gary Shaw's deltaG unfolding for dimers"
	set myFunc($key,parms) 	"r s c d a b P T"

	set key			 		CRYSTAL
	set myFunc($key,name) 	"Single Crystal fit"
	set myFunc($key,note) 	"f(x) = a0 + a1*cos(2x) + a2*sin(2x)"
	set myFunc($key,parms) 	"a0 a1 a2"

	set xcv(MAX_PARM_NUM)	12
}

#####################################################################
proc fnf {x x2} {
	global xcv panvar

	set tcl_precision 9
	set key $panvar(functionKey)

	if { $key == "LINE" } {
		return [expr $panvar(parms,0)*$x + $panvar(parms,1)]
	}

	if { $key == "QUADRATIC" } {
		return [expr $panvar(parms,0)*$x*$x + $panvar(parms,1)*$x + $panvar(parms,2)]
	}

	if { $key == "CUBIC" } {
		return [expr $panvar(parms,0)*$x*$x*$x + $panvar(parms,1)*$x*$x + $panvar(parms,2)*$x + $panvar(parms,3)]
	}

	if { $key == "EXPON" } {
		return [expr $panvar(parms,0) * exp($x*$panvar(parms,1)) + $panvar(parms,2)]
	}

	if { $key == "T1_RELAX" } {
		set term [expr (0 - $x) / $panvar(parms,2)]
		return [expr $panvar(parms,0)*(1.0 - $panvar(parms,1)*exp($term))]
	}

	if { $key == "T2_RELAX" } {
		return [expr $panvar(parms,0) * exp((0 - $x) / $panvar(parms,1))]
	}

	if { $key == "R2_DIMER" } {
		#if { $x == 0 } {
		#	set x 0.00001
		#}
		set b [expr $panvar(parms,1) - $panvar(parms,2)]
		set c [expr 4.0 * $panvar(parms,1) * $x]
		set d [expr sqrt($panvar(parms,0)) * ($panvar(parms,2) - $panvar(parms,1)) * sqrt($panvar(parms,0) + 8.0 * $x)]
		return [expr (($panvar(parms,0) * $b) + $c + $d) / (4.0 * $x)]
	}

	if { $key == "R2X_DIMER" } {
		if { $x == 0 } { set x 0.00001 }
	    set b [expr sqrt($panvar(parms,0)) * sqrt($panvar(parms,0) + 8.0 * $x)]
		set c [expr 0.25 * ($b - $panvar(parms,0))]
		set d [expr 0.125 * ($panvar(parms,0) + (4.0 * $x) - $b)]
		set e [expr $c / $x]
		set f [expr $d / $x]
		return [expr ($e * $panvar(parms,2)) + (2.0 * $f * $panvar(parms,1)) + ($e * $e * $f * $panvar(parms,3) * $panvar(parms,4))]

	}

	if { $key == "SUM_EXPON_1" } {
		set ans [expr $panvar(parms,0)*exp(0 - $x/$panvar(parms,1)) + \
			$panvar(parms,2)*exp(0 - $x/$panvar(parms,3))] 
		return $ans
	}

	if { $key == "SUM_EXPON_2" } {
		set ans [expr $panvar(parms,0)*(1.0 - exp(0 - $x/$panvar(parms,1))) + \
			$panvar(parms,2)*(1.0 - exp(0 - $x/$panvar(parms,3)))] 
		return $ans
	}

	if { $key == "PH_TITR" } {
		set y [expr (0 - $x) * $panvar(parms,3)]
		set b [expr pow(10.0, $y)]
		set y [expr (0 - $panvar(parms,0)) * $panvar(parms,3)]
		set c [expr pow(10.0, $y)]
		set term [expr $b / ($b + $c)]
		set term [expr $b / ($b + $c)]
		return [expr $panvar(parms,1) + $term * ($panvar(parms,2) - $panvar(parms,1))]
	}

	if { $key == "SINU_1" } {
		set b [expr cos(abs($x-60.0)*$xcv(TO_RAD))]
		return [expr $panvar(parms,0) * ($b * $b) - $panvar(parms,1) * $b + $panvar(parms,2)]
	}

	if { $key == "SINU_2" } {
		set b [expr abs($x-60.0)*$xcv(TO_RAD)]
		return [expr $panvar(parms,0) * cos($panvar(parms,1)*$b + $panvar(parms,2)) + $panvar(parms,3)]
	}

	if { $key == "SIGMOID" } {
		set b [expr exp((0.0 - ($x - $panvar(parms,2))) / $panvar(parms,3))]
		return [expr $panvar(parms,0) + $panvar(parms,1) / (1.0 + $b)]
	}

	if { $key == "Z_D_SIG" } {
		set j [expr 8.314 * $panvar(parms,7)]
		set k [expr exp((0.0 - ($panvar(parms,4) + $panvar(parms,5)*$x)) / $j)]
		set m [expr $panvar(parms,2) + $panvar(parms,3) * $x]
		set n [expr ($panvar(parms,0) + $panvar(parms,1) * $x) - $m]
		set p [expr sqrt($k * $k + 8.0 * $panvar(parms,6) * $k)]
		return [expr ($n * ($p - $k)) / (4.0 * $panvar(parms,6)) + $m]
	}

	if { $key == "Z_UCD" } {
		set c [expr $panvar(parms,0) * exp($panvar(parms,1) * $panvar(parms,2) * ($x - $panvar(parms,3)))]
		set a2 [expr $panvar(parms,0) * $panvar(parms,0) * $panvar(parms,4)]
		set d [expr $a2 * exp($panvar(parms,1) * $panvar(parms,2) * 2.0 * ($x - $panvar(parms,3)))]
		return [expr $c + $d]
	}

	if { $key == "Z_M_MELT" } {
		set b [expr $x + 273.15]
		set c [expr $b * (($panvar(parms,4) / $panvar(parms,6)) + $panvar(parms,5) * log($b / $panvar(parms,6)))]
		set d [expr $panvar(parms,4) + $panvar(parms,5) * ($b - $panvar(parms,6))]
		set e [expr ($d - $c) / 0.001987]
		set f [expr exp((0.0 - $e) / $b)]
		set g [expr $f / (1.0 + $f)]
		set term1 [expr $panvar(parms,0) - $panvar(parms,1) * $x]
		set term2 [expr $panvar(parms,2) - $panvar(parms,3) * $x]
		set h [expr (1.0 - $g) * $term1 + $g * $term2]
		return $h 
	}

	if { $key == "Z_D_MELT" } {
		set b [expr $x + 273.15]
		set c [expr $b * (($panvar(parms,4) / $panvar(parms,6)) + $panvar(parms,5) * log($b / $panvar(parms,6)) + 0.001987 * log($panvar(parms,7)))]
		set d [expr $panvar(parms,4) + $panvar(parms,5) * ($b - $panvar(parms,6))]
		set e [expr ($d - $c)]
		set f [expr exp((0.0 - $e) / ($b * 0.001987))]
		set g [expr ((0.0 - $f) + sqrt(($f * $f) + 8.0 * $panvar(parms,7) * $f)) / (4.0 * $panvar(parms,7))]
		set term1 [expr $panvar(parms,0) - $panvar(parms,1) * $x]
		set term2 [expr $panvar(parms,2) - $panvar(parms,3) * $x]
		set h [expr (1.0 - $g) * $term1 + $g * $term2]
		return $h 
	}

	if { $key == "XY1" } {
		set b [expr $x + $x2 + $panvar(parms,1)]
		set sq [expr sqrt($b*$b - 4.0*$x*$x2)]
		set d [expr $panvar(parms,0) * ($b - $sq) / (2.0 * $x2)]
		return $d
	}

	if { $key == "XY2" } {
		set b [expr $panvar(parms,3) - 4.0 * $panvar(parms,1)]
		set c [expr $panvar(parms,3) * $panvar(parms,3)]
		set d [expr ($c - (4.0*$panvar(parms,1)*$panvar(parms,3)) - (2.0*$x2*$panvar(parms,3))) / $b]
		set e [expr ($x*$panvar(parms,3)*(2.0*$x2 - $x) - $c*($x2+$panvar(parms,1)+$x)) / $b]
		set f [expr -(2.0*$d*$d*$d - 9.0*$e*$d + (27.0*$x*$x2*$c / $b)) / 54.0]
		set g [expr (3.0 * $e - $d * $d) / 3.0]
		set w [expr $f / sqrt(-($g*$g*$g) / 27.0)]
		set h [expr $xcv(TO_DEG) * acos($w)]
		set x3 [expr (2.0 * sqrt(-$g/3.0) * cos((($h/3.0)+240)*$xcv(TO_RAD)) - ($d/3.0))]
		set y [expr (($panvar(parms,0)*$x3) + ($panvar(parms,2)*$x3*($x-$x3)/(2*$x3+$panvar(parms,3))))/$x2]
		return $y 
	}

	if { $key == "LORENTZ" } {
		set b [expr 4.0 * $xcv(PI) * $xcv(PI)]
		set v [expr $x - $panvar(parms,0)]
		set lz1 [expr $panvar(parms,1) / (1.0 + $b * $v * $v * $panvar(parms,1) * $panvar(parms,1))] 
		return [expr $panvar(parms,2) * $lz1]
	}

	if { $key == "SUM_LOREN" } {
		set b [expr 4.0 * $xcv(PI) * $xcv(PI)]
		set v [expr $x - $panvar(parms,0)]
		set lz1 [expr $panvar(parms,1) / (1.0 + $b * $v * $v * $panvar(parms,1) * $panvar(parms,1))] 
		set v [expr $x - $panvar(parms,2)]
		set lz2 [expr $panvar(parms,3) / (1.0 + $b * $v * $v * $panvar(parms,3) * $panvar(parms,3))] 
		return [expr 0.0 - $panvar(parms,4) * ($lz1 - $panvar(parms,5) * $lz2)]
	}

	if { $key == "GAUSS" } {
		set b [expr ($x - $panvar(parms,0)) / $panvar(parms,1)]
		set lz1 [expr (1.0 / (2.0 * $xcv(PI) * $panvar(parms,1))) * exp(-0.5 * $b * $b)] 
		set b [expr ($x - $panvar(parms,2)) / $panvar(parms,3)]
		set lz2 [expr (1.0 / (2.0 * $xcv(PI) * $panvar(parms,3))) * exp(-0.5 * $b * $b)] 
		return [expr $panvar(parms,4) * ($lz1 + $panvar(parms,5) * $lz2)]
	}

	if { $key == "Z_DISP" } {
		set b [expr 4.0 * $xcv(PI) * $xcv(PI)]
		set c [expr $x - $panvar(parms,0)]
		set d [expr 2.0 * $xcv(PI) * $panvar(parms,1) * $panvar(parms,1) * $c]
		set lz1 [expr $d / (1.0 + $b * $c * $c * $panvar(parms,1) * $panvar(parms,1))] 
		set e [expr $x - $panvar(parms,2)]
		set f [expr 2.0 * $xcv(PI) * $panvar(parms,3) * $panvar(parms,3) * $e]
		set lz2 [expr $f / (1.0 + $b * $e * $e * $panvar(parms,3) * $panvar(parms,3))] 
		return [expr $panvar(parms,4) * ($lz1 - $panvar(parms,5) * $lz2)]
	}

	if { $key == "Z_AMIDE_PH" } {
		set b [expr pow(10,-$x)]
		set c [expr $panvar(parms,1) * pow(10,-14) / $b]
		set d [expr $panvar(parms,2) * $b]
		set e $panvar(parms,3)
		set f [expr $c + $d + $e]
		return [expr 1.0 / (1.0 + $f * $panvar(parms,0))]
	}

	if { $key == "Z_CHEM_TRI" } {
		set b [expr exp((0.0 - $panvar(parms,4) + $panvar(parms,5) * $x) / $panvar(parms,7))]
		set c [expr abs($b / (3.0 * $panvar(parms,6) * $panvar(parms,6)))]
		set d [expr 0.5 * $c]
		set e [expr sqrt($c * $c * (0.25 + 0.03704 * $c))]
		set g [expr $d + $e]
		set h [expr $d - $e]
		set w [expr pow($g, 0.3333333) - pow(abs($h), 0.3333333)]
		return [expr (1.0 - $w) * ($panvar(parms,0) - $panvar(parms,1) * $x) + $w * ($panvar(parms,2) - $panvar(parms,3) * $x)]
	}

	if { $key == "Z_CHEM_MON_TRI" } {
		set b [expr exp((0.0 - $panvar(parms,4) + $panvar(parms,5) * $x) / $panvar(parms,7))]
		set c [expr abs($b / (3.0 * $panvar(parms,6) * $panvar(parms,6)))]
		set d [expr 0.5 * $c]
		set e [expr sqrt($c * $c * (0.25 + 0.03704 * $c))]
		set g [expr $d + $e]
		set h [expr $d - $e]
		set w [expr pow($g, 0.3333333) - pow(abs($h), 0.3333333)]
		set f [expr exp((0.0 - $panvar(parms,0) + $panvar(parms,1) * $x) / $panvar(parms,7))]
		return [expr (1.0 - $w) * $panvar(parms,2) + (1.0 - $f / (1.0 + $f)) * $panvar(parms,3)]
	}

	if { $key == "Z_CP_MONOMER" } {
		set c [expr $x * 0.008314]
		set d [expr $x * $x * $panvar(parms,7) + $x * $panvar(parms,0) + $panvar(parms,1)]
		set e [expr $x * $x * $panvar(parms,2) + $x * $panvar(parms,3) + $panvar(parms,4)]
		set f [expr (($panvar(parms,4) - $panvar(parms,1))*($x - $panvar(parms,6))) + ((0.50 * ($panvar(parms,3) - $panvar(parms,0))) * (( $x * $x ) - ($panvar(parms,6) * $panvar(parms,6)))) + ((0.3333 * ($panvar(parms,2) - $panvar(parms,8))) * (pow($x,3.0) - pow($panvar(parms,6),3.0)))]
		set s [expr (($panvar(parms,4) - $panvar(parms,1))*(log($x) - log($panvar(parms,6)))) + (($panvar(parms,3) - $panvar(parms,0)) * ($x - $panvar(parms,6))) + ((0.50 * ($panvar(parms,2) - $panvar(parms,7))) * (($x * $x) - ($panvar(parms,6) * $panvar(parms,6))))]
		set g [expr $f + $panvar(parms,5)]
		set h [expr exp(($b * (($panvar(parms,5) / $panvar(parms,6)) + $s) - $g) / $c)]
		set v [expr $h / (1.0 + $h)]
		set w [expr $v * (1.0 - $v) * ($g * $g / ($c * $x))]
		return [expr $d + $w + ($v * $f)]
	}

	if { $key == "Z_CP_DIMER" } {
		set c [expr $x * 0.008314]
		set d [expr $x * $x * $panvar(parms,8) + $x * $panvar(parms,0) + $panvar(parms,1)]
		set e [expr $x * $x * $panvar(parms,2) + $x * $panvar(parms,3) + $panvar(parms,4)]
		set f [expr (($panvar(parms,4) - $panvar(parms,1))*($x - $panvar(parms,6))) + ((0.50 * ($panvar(parms,3) - $panvar(parms,0))) * (( $x * $x ) - ($panvar(parms,6) * $panvar(parms,6)))) + ((0.3333 * ($panvar(parms,2) - $panvar(parms,8))) * (pow($x,3.0) - pow($panvar(parms,6),3.0)))]
		set s [expr (($panvar(parms,4) - $panvar(parms,1))*(log($x) - log($panvar(parms,6)))) + (($panvar(parms,3) - $panvar(parms,0)) * ($x - $panvar(parms,6))) + ((0.50 * ($panvar(parms,2) - $panvar(parms,8))) * (($x * $x) - ($panvar(parms,6) * $panvar(parms,6))))]
		set g [expr 2.0 * ($f + $panvar(parms,5))]
		set h [expr exp(($x * 2.0 * (($panvar(parms,5) / $panvar(parms,6)) + $s + 0.008314 * log(1.66 * $panvar(parms,7))) - $g) / $c)]
		set v [expr (( -1.0 * $h + sqrt($h * $h + 8.0 * $panvar(parms,7) * $h)) / (4.0 * $panvar(parms,7)))]
		set w [expr ($v * (1.0 - $v) / (2.0 - $v)) * (2.0 * $g * $g / ($c * $x))]
		return [expr $d + $w + ($v * ($e - $d))]
	}

	if { $key == "Z_CP_TRIMER" } {
		set c [expr $x * 0.008314]
		set d [expr $x * $x * $panvar(parms,8) + $x * $panvar(parms,0) + $panvar(parms,1)]
		set e [expr $x * $x * $panvar(parms,2) + $x * $panvar(parms,3) + $panvar(parms,4)]
		set f [expr (($panvar(parms,4) - $panvar(parms,1))*($x - $panvar(parms,6))) + ((0.50 * ($panvar(parms,3) - $panvar(parms,0))) * (( $x * $x ) - ($panvar(parms,6) * $panvar(parms,6)))) + ((0.3333 * ($panvar(parms,2) - $panvar(parms,8))) * (pow($x,3.0) - pow($panvar(parms,6),3.0)))]
		set s [expr (($panvar(parms,4) - $panvar(parms,1))*(log($x) - log($panvar(parms,6)))) + (($panvar(parms,3) - $panvar(parms,0)) * ($x - $panvar(parms,6))) + ((0.50 * ($panvar(parms,2) - $panvar(parms,8))) * (($x * $x) - ($panvar(parms,6) * $panvar(parms,6))))]
		set g [expr $f + $panvar(parms,5)]
		set h [expr exp($x * (($panvar(parms,5) / $panvar(parms,6)) + $s + 0.008314 * log(2.09 * $panvar(parms,7)*$panvar(parms,7)) - $g) / $c)]
		set i [expr sqrt(1.0 + ($h / (20.25 * $panvar(parms,7)*$panvar(parms,7))))]
		set j [expr ($h / (6.0 * $panvar(parms,7)*$panvar(parms,7))) * (1.0 + $i)]
		set k [expr ($h / (6.0 * $panvar(parms,7)*$panvar(parms,7))) * (1.0 - $i)]
		set v [expr pow($j, 0.3333333) + pow($k, 0.3333333)]
		set w [expr ($v * (1.0 - $v) / (3.0 - 2 * $v)) * (3.0 * $g * $g / ($c * $x))]
		return [expr $d + $w + ($v * ($e - $d))]
	}

	if { $key == "CRYSTAL" } {
		set b [expr $panvar(parms,1) * cos(2.0 * $x)]
		set c [expr $panvar(parms,2) * sin(2.0 * $x)]
		return [expr $panvar(parms,0) + $b + $c]
	}
}
