#
# main xcrvfit tcl script 
#

#####################################################################
# main program
#
proc xcrvfit_main { } {
	global xcv

	init_consts

	set date [get_date]
	u_log "\n"
	u_log " ***** xcrvfit - $xcv(VERSION) - $date"
	u_log " ***** Installation: $xcv(PROG_HOME)"

	init_fit_funcs
	init_vars
	get_defaults $xcv(fn,userDefaults) $xcv(fn,sysDefaults)

	do_xcrvfit
}

proc init_consts { } {
	global xcv

	set xcv(exe,FASTCRV)		"$xcv(PROG_HOME)/bin/fastcrvfit"
	set xcv(exe,KAY2CRV)		"$xcv(PROG_HOME)/bin/kay2crv"
	set xcv(exe,FP2CRV)			"$xcv(PROG_HOME)/bin/fp2crv"
	set xcv(exe,JFIT2CRV)		"$xcv(PROG_HOME)/bin/jfit2crv"
	set xcv(fn,sysDefaults) 	"$xcv(PROG_HOME)/lib/defaults/$xcv(os).defaults"
	set xcv(fn,desc)   			"$xcv(PROG_HOME)/lib/functions"
	set xcv(dir,help) 			"$xcv(PROG_HOME)/docs"
	set xcv(url,help)			"http://www.bionmr.ualberta.ca/bds/software/"

	# temporary files defined
	# prefer not to use /tmp because of permissions of same-named files 
	# output files from fastcrvfit software. I think i can address this
	# issue later.
	set xcv(fn,resShortTmp)		"$xcv(MY_HOME)/.xcrvfit.out.results1.tmp"
	set xcv(fn,resLongTmp)		"$xcv(MY_HOME)/.xcrvfit.out.results2.tmp"
	set xcv(fn,resGridTmp)		"$xcv(MY_HOME)/.xcrvfit.out.results3.tmp"
	set xcv(fn,dataTmp)			"$xcv(MY_HOME)/.xcrvfit.data.tmp"

	# log file and output files
	set xcv(fn,log)				"$xcv(MY_HOME)/out.${xcv(NAME)}.log"
	set xcv(fn,out_rmse)		"$xcv(MY_HOME)/out.${xcv(NAME)}.rmse"
	set xcv(fn,userDefaults)	"$xcv(MY_HOME)/${xcv(NAME)}.defaults"

	set xcv(PI)					3.14159265
	set xcv(TO_DEG)				[expr 180.0 / $xcv(PI)]
	set xcv(TO_RAD)				[expr $xcv(PI) / 180.0]
	set xcv(X_MAX)				[expr 0x7fffffff]	
	set xcv(X_MIN)				[expr 0.0 - $xcv(X_MAX)]	

	set xcv(grid_winName)		".$xcv(APP)-grid"
	set xcv(mfit_winName)		".$xcv(APP)-mfit"

	set xcv(win,main)			"xcrvfit"
	set xcv(win,help)			"help"
	set xcv(win,results)		"results"
	set xcv(win,saveplot)		"saveplot"
	set xcv(win,savetxt)		"savetext"
	set xcv(win,settings)		"settings"
	set xcv(win,bounded)		"data-range"
	set xcv(win,multplots)		"multiple-plots"
	set xcv(win,overlay)		"data-overlay"
	set xcv(win,gridfit,main)	"gridfit-main"
	set xcv(win,gridfit,calc)	"gridfit-calc"
	set xcv(win,mfit,main)		"mfit-main"
	set xcv(win,mfit,calc)		"mfit-calc"

	set xcv(win,plot,0)			"xcrvfit-plot"
	set xcv(win,plot,1)			"gridfit-select"
	set xcv(win,plot,2)			"gridfit-sse"
	set xcv(win,plot,3)			"gridfit-plot"
	set xcv(win,plot,4)			"mfit-select"
	set xcv(win,plot,5)			"mfit-plot"

	set xcv(data,comment)		"#"
}

proc init_vars { } {
	global myData panvar xcv

	set myData(0,0,iDataNum)		0
	set myData(0,iSetNum)	0

	set panvar(data,filename) 	""
	set panvar(dataSet)			0

	set panvar(parms,num) 	999
	set panvar(dataIndex)	-1	
	set panvar(0,gridSize)	"1 X 1"	
	set panvar(resizeFlag)	"FALSE"	
}

#####################################################################
#
# read the defaults file (look in user's directory and system location)
#
proc get_defaults { f_user f_sys } {

#
# first read system defaults
#
	if ![file readable $f_sys ] {
		u_log "Unable to open installed defaults file: $f_sys"
		puts stderr "Unable to open installed defaults file: $f_sys"
		exit 1
	}

	if [catch {option readfile $f_sys interactive} defaults_err] {
		u_log "Problem reading option in $f_sys: $defaults_err"
		puts stderr "Problem reading option in $f_sys: $defaults_err"
		exit 1
	} else {
		u_log "Reading system defaults file in: '$f_sys'."
	}

#
# then read user defaults
#
	if [file readable $f_user ] {
		u_log "Reading user defaults file in: '$f_user'"
		if [catch {option readfile $f_user interactive} err] {
			u_log " **Warning, problem reading option in $f_user: $err"
			puts stderr "Warning, problem reading option in $f_user: $err"
		}
	} else {
		u_log "No user defaults file."
	}
}

#####################################################################
#
# set defaults if we cannot find the defaults file 
#
proc set_defaults {} {
	global panvar xcv

	set fc .$xcv(win,main)

	set_val panvar(maxIt) 		$fc max_iterations 	500
	set_val panvar(functionKey) $fc function_key	LINE
	set_val xcv(colors) 		$fc app_colors  	{white ivory yellow beige orange pink red green blue skyblue midnightblue brown violet purple gray black}
	set_val xcv(data_symbol_list) 	$fc data_symbol_list  	{rect oval cross diamond}
	set_val xcv(yes_no_list) 	$fc yes_no_list  	{yes no}
}


#####################################################################
#
# See if xcrvfit is being called with arguments 
#
proc check_arguments { } {
	global xcv argc argv panvar

	set panvar(functionKey) LINE

	if { $xcv(os) == "AQUA" } {
		u_log "This looks like a mac osx aqua installation, ignore command line arguments"
		return
	}

	if { $argc == 0  } {
		u_log "No command line arguments to xcrvfit."
		return
	}

	# process datafile
	set arg1 [lindex $argv 0]
	set panvar(data,filename) $arg1
	u_log "Command line argument (data file): '$arg1'"
	bind_dataFile

	# process dataset
	if { $argc >= 2  } { 
		set arg2 [lindex $argv 1]
		u_log "Command line argument (data set label): '$arg2'"
		set panvar(dataLabel) $arg2
		bind_dataSet
	}

	# process function argument 
	if { $argc >= 3  } { 
		set arg3 [lindex $argv 2]
		set panvar(functionKey) $arg3
		u_log "Command line argument (functionKey): '$arg3'"
	}

	# remaining arguments are starting parameters for above function
	if { $argc >= 4  } {
		set pix 0
		for {set i 3} {$i < $argc} {incr i} {
			set panvar(choice,$pix) "no"
			if {[string compare [lindex $argv $i] "const"] == 0} {
				set panvar(choice,$pix) "yes"
				incr i
			}
			set panvar(parms,$pix) [lindex $argv $i]
			set panvar(newParms,$pix) [lindex $argv $i]
			u_log "Command line argument (parameter): '$panvar(parms,$pix)' Hold constant = '$panvar(choice,$pix)'"
			incr pix
		}
	}
}

#####################################################################
#####################################################################
#
# main program
#
proc do_xcrvfit {} {
	global xcv panvar

	set f .$xcv(win,main)
	toplevel $f
	wm title $f "$xcv(NAME)"

	frame $f.frame
	pack $f.frame -fill both -expand true
	set_defaults

	panel_layout $f.frame

	# process user's arguments to the software
	# and set default fitting function
	check_arguments
	bind_fit_func $panvar(functionKey)

	tkwait window $f.frame
}
