
#####################################################################
#
# xcrvfit options window widgets
#
#####################################################################


#####################################################################
# 
# Options menu  
#
proc do_options_menu {f} {
	global panvar xcv

	menu $f.menu

	$f.menu add command -label "Multiple Plots..." \
		-command "multiple_plot"
	$f.menu add command -label "Data Overlays..." \
		-command "data_overlay"
	$f.menu add command -label "Select Data Range..." \
		-command "plot_set_bounds 0 .$xcv(win,bounded)"

	$f.menu add separator

# Add this section in v4.1.x
#	$f.menu add command -label "Multiple Data Fitting Analysis..." \
#		-command "do_mfit"

	$f.menu add command -label "Parameter Grid Fitting..." \
		-command "do_gridfit"
}

#####################################################################
#
# Manage multiple plots 
#
proc multiple_plot { } {
	global xcv panvar vPlot

	set pid 0
	set f .$xcv(win,multplots)
	if [winfo exists $f] {
		raise $f
		return
	}
	toplevel $f

	set g $f.layout
	frame $g
	pack $g -pady 5 -padx 20
	label $g.label -text "Plot Layout: " -anchor e
	menubutton $g.grid -padx 0 -menu $g.grid.menu -width 7 -anchor w \
		-textvar panvar(0,gridSize) -underline 0
    pack $g.label $g.grid -side left
    menu $g.grid.menu
	set panvar(grid_max) 5
	for {set j 1} {$j < $panvar(grid_max)} {incr j} {
		for {set k 1} {$k < $panvar(grid_max)} {incr k} {
        	$g.grid.menu add command -label "$j X $k" \
				-command "do_plot_grid $pid $j $k"
		}
    }
    pack $g.label $g.grid -side left

	set g $f.current
	frame $g
	pack $g -pady 5 -padx 20
	label $g.label -text "Current Plot: " -anchor e
	set pNum $vPlot($pid,pNumCur)
	set vPlot($pid,pNumCurText) $vPlot($pid,$pNum,0,data_label_text)
	label $g.cur -textvar vPlot($pid,pNumCurText) -anchor e
	button $g.next -padx 0 -width 7 -text Next -command "bind_next_plot 1"
	button $g.prev -padx 0 -width 7 -text Prev -command "bind_next_plot -1"
    pack $g.label $g.cur $g.next $g.prev -side left

	set g $f.pbut
	frame $g
	pack $g -pady 5 -padx 20
	button $g.new -text "Add a New Plot" -command "bind_new_plot"
	button $g.remove -text "Remove Current Plot" -command "bind_remove_plot"
	button $g.all -text "Clear Plots, Start Over" -command "plot_reset 0"
    pack $g.new $g.remove $g.all -side left

	frame $f.but
	pack $f.but -pady 10 -padx 20 
	button $f.but.quit -text "Dismiss" -command "destroy $f"
	pack $f.but.quit -side left 
}

#####################################################################
#
# Manage multiple plots 
#
proc data_overlay { } {
	global xcv panvar

	set pid 0
	set f .$xcv(win,overlay)
	if [winfo exists $f] {
		raise $f
		return
	}
	toplevel $f

	set g $f.button
	frame $g
	pack $g -pady 5 -padx 20
	button $g.new -text "New Overlay" -command "bind_new_overlay"
	button $g.next -text "Set Current Data" -command "bind_next_overlay"
    pack $g.new $g.next -side left

	set g $f.but2
	frame $g
	pack $g -pady 5 -padx 20
	button $g.remove -text "Remove Current Overlay" -command "bind_remove_plot"
	button $g.all -text "Clear Plots, Start Over" -command "plot_reset 0"
    pack $g.remove $g.all -side left

	frame $f.but
	pack $f.but -pady 10 -padx 20 
	button $f.but.quit -text "Dismiss" -command "destroy $f"
	pack $f.but.quit -side left 
}

#####################################################################
# 
# Set new plot window grid 
#
proc do_plot_grid { pid row col } {
	global xcv panvar vPlot

	set vPlot($pid,grid,row) $row
	set vPlot($pid,grid,col) $col
	set panvar($pid,gridSize) "$row X $col"

	plot_set_grid $pid $vPlot($pid,grid,row) $vPlot($pid,grid,col)

	for {set j 0} {$j < $vPlot($pid,pNumTot)} {incr j} {
		if { $vPlot($pid,$j,okFlag) == "TRUE" } {
			plot_init_new $pid $pid $j 0
		}
	}
	do_plot $pid 0
}

#####################################################################
#
# user wants to overlay (keep) this data on the current plot 
#
proc bind_next_overlay { } {
	global vPlot xcv xwin panvar myData

	set pid 0 
	set pNum $vPlot($pid,pNumCur)
	set dNum $vPlot($pid,$pNum,dNumCur)

	incr dNum
	if { $dNum >= $vPlot($pid,$pNum,dNumTot) } {
		set dNum 0
	}
	set vPlot($pid,$pNum,dNumCur) $dNum

	blink_data $pid $pNum $dNum 100
}

#####################################################################
#
# Make the next or previous plot current 
#
proc bind_next_plot { inc } {
	global vPlot xcv xwin panvar myData

	set pid 0 
	set flag FALSE 
	set start $vPlot($pid,pNumCur)

	while { $flag == "FALSE" } {
		set pNum [expr $vPlot($pid,pNumCur) + $inc]
		if { $pNum >= $vPlot($pid,pNumTot) } {
			set pNum 0
		}
		if { $pNum < 0 } {
			set pNum [expr $vPlot($pid,pNumTot) - 1] 
		}
		set flag $vPlot($pid,$pNum,okFlag)
		if {$pNum == $start} {break}
		set vPlot($pid,pNumCur) $pNum
	}

	set vPlot($pid,pNumCur) $pNum
	set vPlot($pid,$pNum,dNumCur) 0
	set vPlot($pid,pNumCurText) $vPlot($pid,$pNum,0,data_label_text)

	blink_data $pid $pNum 0 100
}

#####################################################################
#
# user wants to overlay (keep) this data on the current plot 
#
proc bind_new_overlay { } {
	global vPlot xcv xwin panvar myData

	set pid 0 
	set pNum $vPlot($pid,pNumCur)
	set dNum $vPlot($pid,$pNum,dNumCur)

	if ![ info exists vPlot($pid,$pNum,$dNum,mix) ] {
		return
	}

	# check for current dataset 
	if {$panvar(dataIndex) < 0} {
		u_error "Unable to save this plot, Data File not read."
	}

	set dNum $vPlot($pid,$pNum,dNumTot)
	set vPlot($pid,$pNum,dNumCur) $dNum
	incr vPlot($pid,$pNum,dNumTot)

	bind_next_dataSet 1
}

#####################################################################
# 
# remove current plot Axis 
#
proc bind_remove_overlay { } {
	global vPlot panvar

	set pid 0 
	u_error " This feature to be implemented in a future version"
	if {$vPlot($pid,pNumTot) <= 1} {
		return
	}

#	set pNum $vPlot($pid,pNumCur)
#	set vPlot($pid,$pNum,drawFlag) "FALSE"
#	do_plot $pid 0
}

#####################################################################
# 
# Create new plot Axis 
#
proc bind_new_plot { } {
	set pid 0 
	set iFile 0
	set mix [get_next_dataSet 1 $iFile]
	plot_add $pid $mix
	bind_next_plot 1
}

#####################################################################
# 
# remove current plot Axis 
#
proc bind_remove_plot { } {
	global vPlot panvar

	if { $vPlot(valid,total) <= 1 } {
		u_error "You cannot delete the last plot."
	}

	set pid 0
	set iFile 0
	set mix [get_next_dataSet -1 $iFile]
	plot_remove $pid $mix
	bind_next_plot -1
}

#####################################################################
# 
# Determine the layout of our multiple plots 
#
proc create_plot_grid { pid start } {
	global vPlot panvar

	set plotNum $start 
	set vPlot($pid,grid,row) 1
	set vPlot($pid,grid,col) 1
	set totPlot 1
	for {set j 0} {$j < $vPlot($pid,pNumTot)} {incr j} {
		if { $vPlot($pid,$j,okFlag) == "TRUE" } {
			incr plotNum
		}
		if { $plotNum > $totPlot } {
			if {$vPlot($pid,grid,row) >= $vPlot($pid,grid,col)} {
				incr vPlot($pid,grid,col)
			} else {
				incr vPlot($pid,grid,row)
			}
			set totPlot [expr $vPlot($pid,grid,row) * $vPlot($pid,grid,col)] 
		}
	}
	set panvar($pid,gridSize) "$vPlot($pid,grid,row) X $vPlot($pid,grid,col)"
	plot_set_grid $pid $vPlot($pid,grid,row) $vPlot($pid,grid,col)
}

#####################################################################
#####################################################################
#####################################################################


#####################################################################
# 
# Set new plot window grid 
#
proc next_plot { cVar inc } {
	global xcv vPlot panvar

	set n [expr $panvar($cVar) + $inc]
	if {$n > $panvar(grid_max)} {
		set n 1
	}
	if {$n <= 0} {
		set n $panvar(grid_max)
	}
	set var panvar($cVar)
	set $var $n
	set pid 0
	set vPlot($pid,pNumCur) $n
}

#####################################################################
# 
# Set new plot window grid 
#
proc current_plot { inc } {
	global xcv panvar

	do_plot $xcv(win,plot) 0
}


proc pop_max_it {f} {
	global panvar

	toplevel $f
	label $f.nop1
	label $f.nop2

	command_entry $f.max " Max Iterations" 15 \
		"destroy $f" -textvar panvar(maxIt) -width 9 
	command_entry $f.offset  "    X-Offset" 12 {bind_offset} \
		-textvar panvar(xOffset) -width 6
	pack $f.nop1 $f.max $f.offset $f.nop2 -side top

	button $f.but -text "Cancel" -command "destroy $f"
	pack $f.but -side top
}

#####################################################################
#
# event procedure for changing the X-offset field 
# 
proc bind_offset {} {
	global myData panvar 

	u_log "X-offset is '$panvar(xOffset)'."
	for {set j 0} {$j < $myData(0,iSetNum)} {incr j} {
		set ub [expr $myData(0,$j,iDataNum)]
		for {set i 0} {$i < $ub} {incr i} {
			set myData(0,$j,$i,xd) [expr $myData(0,$j,$i,xdOrig) + $panvar(xOffset)]
		}
	}
	bind_next_dataSet 0
}
