
#####################################################################
#
# utility functions
#
#####################################################################

#
# set a variable from the defaults file
#
proc set_val {var struc entry val} {
	global panvar

	upvar $var a
	if ![info exists a] {
		set a [option get $struc $entry {}]
		if {$a == ""} { set a $val }
	}
	set panvar($entry) $a
}

#
# load file into text widget
#
proc load_file { head file1 wd ht} {
	global xcv
	set f1 $xcv(win,main).$head
	toplevel $f1

	set tf $f1.msg
	frame $f1.msg

    text $tf.text -relief raised -bd 2 -font courier -width $wd -state normal -height $ht \
		-yscrollcommand "$tf.scroll set"
	scrollbar $tf.scroll -command "$tf.text yview"
	pack $tf.scroll -side right -fill y
	pack $tf.text -side left

    button $f1.but -text "Quit" -command "destroy $f1"
	pack $f1.msg $f1.but

	$tf.text delete 1.0 end
	set f [open $file1]
	while {![eof $f]} {
		$tf.text insert end [read $f 1000]
	}
	close $f
}

proc get_max {x y} {
	if {$x > $y} {return $x}
	return $y
}

proc get_min {x y} {
	if {$x > $y} {return $y}
	return $x
}

######################################################################
#
# generic file entry widget
#
proc u_file_entry { f title tWidth tVar eWidth padx cmd browse} {
    pack [frame $f] -fill x -expand true -padx 1
    pack [frame $f.a] -side left
    pack [frame $f.b] -side left -fill x -expand true
    pack [frame $f.c] -side left
	label $f.a.label -text $title -anchor e -width $tWidth
	pack $f.a.label -side right
	entry $f.b.entry -bd 2 -textvariable $tVar -width $eWidth -relief sunken
#	$f.b.entry config -cursor {xterm white}
	bind $f.b.entry <Return> $cmd
	pack $f.b.entry -side left -fill x -expand true
	if { $browse > 0 } {
		if { $browse == 1 } {
			button $f.c.browse -text Browse \
				-padx $padx \
				-command "u_file_browse $tVar $cmd" 
		} else {
			button $f.c.browse -text Browse \
				-padx $padx \
				-command "u_dir_browse $tVar" 
		}
		pack $f.c.browse -side left -padx 8
	}
}

######################################################################
#
# generic command field entry
#
proc u_command_entry {name label width command args} {
	set f $name.box
    pack [frame $f] -fill x -expand true -padx 1
	label $f.label -text $label -width $width -anchor e
	eval { entry $f.entry -relief sunken -bd 2 } $args
	pack $f.label -side left
	pack $f.entry -side right -fill x -expand true
	bind $f.entry <Return> $command
	return $f.entry
}

# 
# generic command field entry
#
proc command_entry {name label width command args} {
	frame $name
	label $name.label -text $label -width $width -anchor e
	eval {entry $name.entry -relief sunken -bd 2 } $args
	pack $name.label -side left 
	pack $name.entry -side left -fill x -expand true
	bind $name.entry <Return> $command
	return $name.entry
}

######################################################################
#
# purpose: to call a tk routine to search for a file 
#
proc u_file_browse { fn cmd } {
	global panvar

	set chosen [tk_getOpenFile]
	u_log "Clicked Browse button to select file: '$chosen'."
	if { $chosen != "" } { 
		set currentpwd [pwd] 
		set cmd2 "regsub {$currentpwd\/} $chosen \"\" shortchosen"
		eval $cmd2
		set $fn $shortchosen
		$cmd
	}
}

######################################################################
#
# purpose: to call a tk routine to search for a directory 
#
proc u_dir_browse { dir } {
	global dataDir 
	set chosen [tk_chooseDirectory]
	if { $chosen != "" } { 
		set currentpwd [pwd]
		set cmd "regsub {$currentpwd\/} $chosen \"\" shortchosen"
		eval $cmd
		set $dir $shortchosen
	}
}

######################################################################
#   
# create a generic widget for file text display
#       
proc u_textwin { f width height } {
	frame $f
	pack $f -expand true -fill both -padx 10 -pady 10
    text $f.textwin -relief raised -bd 2 \
		-width $width -height $height \
		-setgrid false -wrap none \
		-xscrollcommand [list $f.xscroll set] \
		-yscrollcommand [list $f.yscroll set]
	scrollbar $f.xscroll -orient horizontal \
		-command [list $f.textwin xview]
	scrollbar $f.yscroll -orient vertical \
		-command [list $f.textwin yview]
	pack $f.xscroll -side bottom -fill x
	pack $f.yscroll -side right -fill y
	pack $f.textwin -side left -fill both -expand true
}

######################################################################
#   
# loads a file onto a canvas - code taken form "TCL and the TK toolkit"
#       
proc u_load_file { file textItem } {
    $textItem delete 1.0 end
    if [catch {open $file r} f] {
		$textItem insert end "Not able to read selected file."
		return -1
	}
	while {![eof $f]} {
		$textItem insert end [read $f 10000]
	}
	close $f
}   

######################################################################
#
# raise existing window on desktop or create it if it does not exist
#
proc pop_question { msg } {
	global panvar

	set f .confirm
 	toplevel $f
	frame $f.win
	frame $f.buttons
	pack $f.win -fill both -expand true -pady 10
	pack $f.buttons

	text $f.win.t -setgrid false -wrap word -width 60 -height 5 -relief flat
	$f.win.t insert end $msg
	pack $f.win.t -padx 20 -pady 20 -fill both -expand true

	button $f.buttons.yes -text "YES" -padx 10 -width 5 \
		-command {set panvar(response) 1}
	button $f.buttons.no -text "NO" -padx 10 -width 5 \
		-command {set panvar(response) 0}
	pack $f.buttons.yes $f.buttons.no -side left

	tkwait variable panvar(response)

	destroy $f
	if {$panvar(response)} {
		return 0
	}
	return 1
}

######################################################################
#
# raise existing window on desktop or create it if it does not exist
#
proc u_pop_win { f title } {
	# if window exists, make sure user can see it and return
	if [winfo exists $f] {
		set wState [wm state $f]
		if { $wState == "iconic" } {
			wm deiconify $f
		}
		raise $f
		return "FALSE"
	}

	# window does not exist, we should create it
	toplevel $f
	wm title $f $title
	return TRUE 
}

######################################################################
#
# put entry in log file 
#
proc save_text {tf outfile} {

	set f [open $outfile w]
	set txt [$tf get 1.0 end] 
	puts $f $txt
	u_log $txt
	close $f
}

######################################################################
#
# put entry in log file 
#
proc u_log {msg} {
	global xcv

	set f [open $xcv(fn,log) a]
	puts $f $msg
#	puts stdout $msg
	close $f
}

######################################################################
#
#  my error procedure 
#
proc u_error {msg} {

#	u_log " **Error: $msg"
	u_log " **Error: $msg"
	error $msg
}

#
# update defaults file with user selection
#
proc write_parms {} {
    global xcv

	# This can be complicated to maintain
	# I think i will do a re-write of this section later.
	# For now, let user take care of personal defaults for xcrvfit.
	return	

	# defaults file does not exist
	if [catch {open $xcv(fn,userDefaults) r} fileId] {
		write_xcrvfit_parms
		return
	}

	# read user defaults until we come to user defined section.
	set infp [open $xcv(fn,userDefaults) r]
	while {[gets $fileId rline] >= 0} {
		set nd [scan $rline "%s %s" comment key]
		if { $nd > 1 } {
			if {[string compare $key "START_SECTION:"] == 0} {
				break
			}
		}
	}

	# get those user settings
	set count 0
	while {[gets $fileId rline] >= 0} {
		set userData($count) $rline
		incr count
	}
	close $fileId

	write_xcrvfit_parms

	# add user defaults to the end of current settings.
	if { $count > 0 } {
		set fpo [open $xcv(fn,userDefaults) a]
		for {set i 0} {$i < $count} {incr i} {
			puts $fpo $userData($i)
		}
		close $fpo
	}
}

#
# update defaults file with user selection
#
proc write_xcrvfit_parms {} {
    global xcv panvar

	set fpo [open $xcv(fn,userDefaults) w]
	puts $fpo "####################################################### "
	puts $fpo "# "
	puts $fpo "# User defaults file for xcrvfit v4.x and higher"
	puts $fpo "# "
	puts $fpo "####################################################### "
	puts $fpo "# "
	puts $fpo "# The following parameters are set by xcrvfit software."
	puts $fpo "# Do not edit entries in this section."
	puts $fpo "# "

	write_plot_defaults $fpo

	puts $fpo "# "
	puts $fpo "####################################################### "
	puts $fpo "# "
	puts $fpo "# START_SECTION: Append user specified defaults after this line."
	close $fpo
}
